"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
function getJSXIdentifier(node, tag = false) {
    if (core_1.types.isIdentifier(node) && (!tag || node.name.match(/^[A-Z]/))) {
        return core_1.types.jsxIdentifier(node.name);
    }
    if (core_1.types.isStringLiteral(node)) {
        return core_1.types.jsxIdentifier(node.value);
    }
    return null;
}
function getJSXAttributeValue(node) {
    if (core_1.types.isStringLiteral(node)) {
        return node;
    }
    if (core_1.types.isJSXElement(node)) {
        return node;
    }
    if (core_1.types.isExpression(node)) {
        return core_1.types.jsxExpressionContainer(node);
    }
    return null;
}
/**
 * Tests if a node is `null` or `undefined`
 */
const isNullLikeNode = (node) => core_1.types.isNullLiteral(node) || core_1.types.isIdentifier(node, { name: 'undefined' });
/**
 * Tests if a node is an object expression with noncomputed, nonmethod attrs
 */
const isPlainObjectExpression = (node) => core_1.types.isObjectExpression(node) &&
    node.properties.every(property => core_1.types.isSpreadElement(property) ||
        (core_1.types.isObjectProperty(property, { computed: false }) &&
            getJSXIdentifier(property.key) != null &&
            getJSXAttributeValue(property.value) != null));
/**
 * Tests if a node is a CallExpression with callee `React.createElement`
 */
const isReactCreateElement = (node) => core_1.types.isCallExpression(node) &&
    core_1.types.isMemberExpression(node.callee) &&
    core_1.types.isIdentifier(node.callee.object, { name: 'React' }) &&
    core_1.types.isIdentifier(node.callee.property, { name: 'createElement' }) &&
    !node.callee.computed;
/**
 * Tests if a node is a MemberExpression with object `Object.assign`
 */
const isObjectAssignMemberExpression = (node) => core_1.types.isCallExpression(node) &&
    core_1.types.isMemberExpression(node.callee) &&
    core_1.types.isIdentifier(node.callee.object, { name: 'Object' }) &&
    core_1.types.isIdentifier(node.callee.property, { name: 'assign' });
/**
 * Get a array of JSX(Spread)Attribute from a props ObjectExpression.
 * Handles the _extends Expression babel creates from SpreadElement nodes.
 * Returns null if a validation error occurs.
 */
function getJSXProps(node) {
    if (node == null || isNullLikeNode(node)) {
        return [];
    }
    if (core_1.types.isCallExpression(node) &&
        core_1.types.isIdentifier(node.callee, { name: '_extends' })) {
        const props = node.arguments.map(getJSXProps);
        // if calling this recursively works, flatten.
        if (props.every(prop => prop != null)) {
            return [].concat(...props);
        }
    }
    if (!core_1.types.isObjectExpression(node) && core_1.types.isExpression(node))
        return [core_1.types.jsxSpreadAttribute(node)];
    if (!isPlainObjectExpression(node)) {
        return null;
    }
    return node.properties.map((prop) => core_1.types.isObjectProperty(prop)
        ? core_1.types.jsxAttribute(getJSXIdentifier(prop.key), getJSXAttributeValue(prop.value))
        : core_1.types.jsxSpreadAttribute(prop.argument));
}
/**
 * Apply className to properties of an ObjectExpression
 */
function traverseApplyClassName(nodeArgs, className) {
    nodeArgs &&
        nodeArgs.forEach(arg => {
            if (core_1.types.isObjectExpression(arg)) {
                if (arg.properties) {
                    const classNameAttrIndex = arg.properties.findIndex(attr => 'key' in attr &&
                        'name' in attr.key &&
                        attr.key.name === 'className');
                    // if className is already set, add the className to the existing className
                    if (classNameAttrIndex !== -1) {
                        const classNameAttr = arg.properties[classNameAttrIndex];
                        if (classNameAttr.value &&
                            classNameAttr.value.value &&
                            !classNameAttr.value.value.split(' ').includes(className)) {
                            classNameAttr.value.value = `${classNameAttr.value.value} ${className}`;
                        }
                    }
                    else {
                        // push object property
                        arg.properties.push(core_1.types.objectProperty(core_1.types.identifier('className'), core_1.types.stringLiteral(className)));
                    }
                }
                return;
            }
            if (core_1.types.isCallExpression(arg)) {
                if (isReactCreateElement(arg)) {
                    traverseApplyClassName(arg.arguments, className);
                }
                if (isObjectAssignMemberExpression(arg)) {
                    // only parse the first ObjectExpression argument
                    const parseIdx = arg.arguments.findIndex(a => core_1.types.isObjectExpression(a));
                    traverseApplyClassName([arg.arguments[parseIdx]], className);
                }
            }
        });
}
function CallExpression(path, { opts }) {
    const customClassName = Array.isArray(opts.className)
        ? opts.className.join(' ')
        : opts.className;
    const node = path.node;
    if (!isReactCreateElement(node)) {
        return;
    }
    const [, propsNode] = node.arguments;
    const props = getJSXProps(propsNode);
    if (props) {
        traverseApplyClassName(node.arguments, customClassName);
    }
}
exports.default = CallExpression;
